use reqwest::StatusCode;

pub type Result<T> = std::result::Result<T, Error>;

#[derive(Debug, thiserror::Error)]
pub enum Error {
  #[error("not found")]
  NotFound,
  #[error("lodestone responded with an unexpected code: {0}")]
  UnexpectedResponse(StatusCode),
  #[error("network error: {0}")]
  Net(reqwest::Error),
  #[error("url parse error: {0}")]
  Url(url::ParseError),
  #[error("lodestone parse error: {0}")]
  Parse(lodestone_parser::error::Error),
}
